@@ -1,13 +1,11 @@
-"""
-The `contract` command group
-"""
-
 import json
-from typing import List, Optional, Tuple, cast
+from typing import Any, List, Optional, Tuple, cast
 
-from click import ClickException, Path, argument, command, group, option
+from click import ClickException, Path, argument, group, option
 from web3.contract.contract import ContractFunction
 
+from autonity_cli.auth import validate_authenticator_account
+
 from ..abi_parser import (
     find_abi_constructor,
     find_abi_function,
@@ -17,8 +15,7 @@ from ..abi_parser import (
 from ..logging import log
 from ..options import (
     contract_options,
-    from_option,
-    keyfile_option,
+    from_options,
     rpc_endpoint_option,
     tx_aux_options,
     tx_value_option,
@@ -27,20 +24,15 @@ from ..utils import (
     contract_address_and_abi_from_args,
     create_contract_tx_from_args,
     finalize_tx_from_args,
-    from_address_from_argument,
     to_json,
     web3_from_endpoint_arg,
 )
 
-# pylint: disable=too-many-arguments
-# pylint: disable=too-many-locals
-# pylint: disable=import-outside-toplevel
-
 
 @group(name="contract")
 def contract_group() -> None:
     """
-    Command for interacting with arbitrary contracts.
+    Commands for interacting with arbitrary contracts.
     """
 
 
@@ -50,9 +42,9 @@ def function_call_from_args(
     contract_abi_path: Optional[str],
     method: str,
     parameters: List[str],
-) -> Tuple:
+) -> Tuple[Any, ...]:
     """
-    Take command line arguments and construct a ContractFunction (i.e. a function call).
+    Construct a function call from command line arguments.
 
     Returns the ContractFunction object, the ABIFunction for the method,
     and the Web3 object created in the process.
@@ -73,15 +65,14 @@ def function_call_from_args(
     contract = w3.eth.contract(address, abi=abi)
     contract_fn = getattr(contract.functions, method, None)
     if contract_fn is None:
-        raise ClickException(f"method '{method}' not found on contract abi")
+        raise ClickException(f"Method '{method}' not found on contract ABI")
 
     return contract_fn(*fn_params), abi_fn, w3
 
 
-@command(name="deploy")
+@contract_group.command(name="deploy")
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_value_option()
 @tx_aux_options
 @option(
@@ -89,7 +80,7 @@ def function_call_from_args(
     "contract_path",
     required=True,
     type=Path(),
-    help="Path to JSON file holding contact abi and bytecode",
+    help="path to JSON file holding the contact ABI and bytecode.",
 )
 @argument("parameters", nargs=-1)
 def deploy_cmd(
@@ -97,6 +88,7 @@ def deploy_cmd(
     keyfile: Optional[str],
     from_str: Optional[str],
     contract_path: str,
+    trezor: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
     max_priority_fee_per_gas: Optional[str],
@@ -111,7 +103,7 @@ def deploy_cmd(
     Deploy a contract, given the compiled JSON file.
 
     Note that the contract's address will appear in the 'contractAddress' field of
-    the transaction receipt (see aut tx wait).
+    the transaction receipt (see `aut tx wait`).
     """
 
     log(f"parameters: {list(parameters)}")
@@ -129,7 +121,7 @@ def deploy_cmd(
     log(f"fn_params (parsed): {fn_params}")
     deploy_fn = cast(ContractFunction, contract.constructor(*fn_params))
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
 
     deploy_tx = create_contract_tx_from_args(
         function=deploy_fn,
@@ -150,10 +142,7 @@ def deploy_cmd(
     print(to_json(tx))
 
 
-contract_group.add_command(deploy_cmd)
-
-
-@command(name="call")
+@contract_group.command(name="call")
 @rpc_endpoint_option
 @contract_options
 @argument("method")
@@ -182,13 +171,9 @@ def call_cmd(
     print(to_json(parsed_result))
 
 
-contract_group.add_command(call_cmd)
-
-
-@command(name="tx")
+@contract_group.command(name="tx")
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @contract_options
 @tx_value_option()
 @tx_aux_options
@@ -197,6 +182,7 @@ contract_group.add_command(call_cmd)
 def tx_cmd(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     contract_address_str: Optional[str],
     contract_abi_path: Optional[str],
@@ -225,7 +211,7 @@ def tx_cmd(
         parameters,
     )
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     log(f"from_addr: {from_addr}")
 
     tx = create_contract_tx_from_args(
@@ -245,6 +231,3 @@ def tx_cmd(
 
     tx = finalize_tx_from_args(w3, rpc_endpoint, tx, from_addr)
     print(to_json(tx))
-
-
-contract_group.add_command(tx_cmd)
