@@ -1,17 +1,15 @@
-"""
-The `gov` command group.
-"""
-
 from typing import Optional
 
-from click import argument, command, group
+from autonity.contracts.autonity import Eip1559
+from click import argument, group
 from web3 import Web3
 
-from ..options import from_option, keyfile_option, rpc_endpoint_option, tx_aux_options
+from autonity_cli.auth import validate_authenticator_account
+
+from ..options import from_options, rpc_endpoint_option, tx_aux_options
 from ..utils import (
     autonity_from_endpoint_arg,
     create_contract_tx_from_args,
-    from_address_from_argument,
     parse_newton_value_representation,
     parse_wei_representation,
     to_json,
@@ -25,10 +23,9 @@ def governance_group() -> None:
     """
 
 
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("schedule-vault", metavar="ADDRESS")
 @argument("amount", type=int)
@@ -37,6 +34,7 @@ def governance_group() -> None:
 def create_schedule(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -57,7 +55,7 @@ def create_schedule(
     """
 
     vault_address = Web3.to_checksum_address(schedule_vault)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -74,65 +72,15 @@ def create_schedule(
     print(to_json(tx))
 
 
-governance_group.add_command(create_schedule)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
-@tx_aux_options
-@argument("base-fee-str", metavar="base-fee", nargs=1)
-def set_minimum_base_fee(
-    rpc_endpoint: Optional[str],
-    keyfile: Optional[str],
-    from_str: Optional[str],
-    gas: Optional[str],
-    gas_price: Optional[str],
-    max_priority_fee_per_gas: Optional[str],
-    max_fee_per_gas: Optional[str],
-    fee_factor: Optional[float],
-    nonce: Optional[int],
-    chain_id: Optional[int],
-    base_fee_str: str,
-) -> None:
-    """
-    Set the minimum base fee.
-
-    Restricted to the operator account.
-    See `setMinimumBaseFee` on Autonity contract.
-    """
-
-    base_fee = parse_wei_representation(base_fee_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
-    aut = autonity_from_endpoint_arg(rpc_endpoint)
-
-    tx = create_contract_tx_from_args(
-        function=aut.set_minimum_base_fee(base_fee),
-        from_addr=from_addr,
-        gas=gas,
-        gas_price=gas_price,
-        max_fee_per_gas=max_fee_per_gas,
-        max_priority_fee_per_gas=max_priority_fee_per_gas,
-        fee_factor=fee_factor,
-        nonce=nonce,
-        chain_id=chain_id,
-    )
-    print(to_json(tx))
-
-
-governance_group.add_command(set_minimum_base_fee)
-
-
-@command()
-@rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("duration", type=int, nargs=1)
 def set_max_schedule_duration(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -144,13 +92,13 @@ def set_max_schedule_duration(
     duration: int,
 ) -> None:
     """
-    Set the max allowed duration of any schedule or contract.
+    Set the maximum allowed duration of any schedule or contract.
 
     Restricted to the operator account.
     See `setMaxScheduleDuration` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -167,18 +115,15 @@ def set_max_schedule_duration(
     print(to_json(tx))
 
 
-governance_group.add_command(set_max_schedule_duration)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("committee-size", type=int, nargs=1)
 def set_committee_size(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -195,7 +140,7 @@ def set_committee_size(
     Restricted to the Operator account. See `setCommitteeSize` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -212,18 +157,15 @@ def set_committee_size(
     print(to_json(tx))
 
 
-governance_group.add_command(set_committee_size)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("unbonding-period", type=int, nargs=1)
 def set_unbonding_period(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -240,7 +182,7 @@ def set_unbonding_period(
     Restricted to the Operator account. See `setUnbondingPeriod` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -257,18 +199,15 @@ def set_unbonding_period(
     print(to_json(tx))
 
 
-governance_group.add_command(set_unbonding_period)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("proposer-reward-rate", type=int)
 def set_proposer_reward_rate(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -286,7 +225,7 @@ def set_proposer_reward_rate(
     See `setProposerRewardRate` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -303,18 +242,15 @@ def set_proposer_reward_rate(
     print(to_json(tx))
 
 
-governance_group.add_command(set_proposer_reward_rate)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("oracle-reward-rate", type=int)
 def set_oracle_reward_rate(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -331,7 +267,7 @@ def set_oracle_reward_rate(
     Restricted to the Operator account. See `setOracleRewardRate` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -348,18 +284,15 @@ def set_oracle_reward_rate(
     print(to_json(tx))
 
 
-governance_group.add_command(set_oracle_reward_rate)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("withholding_threshold", type=int)
 def set_withholding_threshold(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -377,7 +310,7 @@ def set_withholding_threshold(
     See `setWithholdingThreshold` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -394,18 +327,15 @@ def set_withholding_threshold(
     print(to_json(tx))
 
 
-governance_group.add_command(set_withholding_threshold)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
-@argument("pool-address-str", type=int)
+@argument("pool-address-str", metavar="POOL-ADDRESS")
 def set_withheld_rewards_pool(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -424,7 +354,7 @@ def set_withheld_rewards_pool(
     """
 
     pool_address = Web3.to_checksum_address(pool_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -441,18 +371,15 @@ def set_withheld_rewards_pool(
     print(to_json(tx))
 
 
-governance_group.add_command(set_withheld_rewards_pool)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("epoch-period", type=int, nargs=1)
 def set_epoch_period(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -469,7 +396,7 @@ def set_epoch_period(
     Restricted to the Operator account. See `setEpochPeriod` on Autonity contract.
     """
 
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -486,18 +413,15 @@ def set_epoch_period(
     print(to_json(tx))
 
 
-governance_group.add_command(set_epoch_period)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("operator-address-str", metavar="OPERATOR-ADDRESS", nargs=1)
 def set_operator_account(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -515,7 +439,7 @@ def set_operator_account(
     """
 
     operator_address = Web3.to_checksum_address(operator_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -532,18 +456,15 @@ def set_operator_account(
     print(to_json(tx))
 
 
-governance_group.add_command(set_operator_account)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("treasury-address-str", metavar="treasury-address", nargs=1)
 def set_treasury_account(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -561,7 +482,7 @@ def set_treasury_account(
     """
 
     treasury_address = Web3.to_checksum_address(treasury_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -578,18 +499,15 @@ def set_treasury_account(
     print(to_json(tx))
 
 
-governance_group.add_command(set_treasury_account)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("treasury-fee-str", metavar="TREASURY-FEE", nargs=1)
 def set_treasury_fee(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -607,7 +525,7 @@ def set_treasury_fee(
     """
 
     treasury_fee = parse_wei_representation(treasury_fee_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -624,18 +542,15 @@ def set_treasury_fee(
     print(to_json(tx))
 
 
-governance_group.add_command(set_treasury_fee)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_accountability_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -647,14 +562,14 @@ def set_accountability_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the Accountability Contract address.
+    Set the Accountability contract address.
 
     Restricted to the Operator account.
     See `setAccountabilityContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -671,18 +586,15 @@ def set_accountability_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_accountability_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_oracle_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -694,13 +606,13 @@ def set_oracle_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the Oracle Contract address.
+    Set the Oracle contract address.
 
     Restricted to the Operator account. See `setOracleContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -717,18 +629,15 @@ def set_oracle_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_oracle_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_acu_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -740,13 +649,13 @@ def set_acu_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the ACU Contract address.
+    Set the ACU contract address.
 
     Restricted to the Operator account. See `setAcuContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -763,18 +672,15 @@ def set_acu_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_acu_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_supply_control_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -786,14 +692,14 @@ def set_supply_control_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the Supply Control Contract address.
+    Set the Supply Control contract address.
 
     Restricted to the Operator account.
     See `setSupplyControlContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -810,18 +716,15 @@ def set_supply_control_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_supply_control_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_stabilization_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -833,14 +736,14 @@ def set_stabilization_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the Supply Control Contract address.
+    Set the Stabilization contract address.
 
     Restricted to the Operator account.
     See `setSupplyControlContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -857,18 +760,15 @@ def set_stabilization_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_stabilization_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_inflation_controller_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -880,14 +780,14 @@ def set_inflation_controller_contract(
     contract_address_str: str,
 ) -> None:
     """
-    Set the inflation controller contract address.
+    Set the Inflation Controller contract address.
 
     Restricted to the Operator account.
     See `setInflationControllerContract` on Autonity contract.
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -904,65 +804,15 @@ def set_inflation_controller_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_inflation_controller_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
-@tx_aux_options
-@argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
-def set_upgrade_manager_contract(
-    rpc_endpoint: Optional[str],
-    keyfile: Optional[str],
-    from_str: Optional[str],
-    gas: Optional[str],
-    gas_price: Optional[str],
-    max_priority_fee_per_gas: Optional[str],
-    max_fee_per_gas: Optional[str],
-    fee_factor: Optional[float],
-    nonce: Optional[int],
-    chain_id: Optional[int],
-    contract_address_str: str,
-) -> None:
-    """
-    Set the upgrade manager contract address.
-
-    Restricted to the Operator account.
-    See `setUpgradeManagerContract` on Autonity contract.
-    """
-
-    contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
-    aut = autonity_from_endpoint_arg(rpc_endpoint)
-
-    tx = create_contract_tx_from_args(
-        function=aut.set_upgrade_manager_contract(contract_address),
-        from_addr=from_addr,
-        gas=gas,
-        gas_price=gas_price,
-        max_fee_per_gas=max_fee_per_gas,
-        max_priority_fee_per_gas=max_priority_fee_per_gas,
-        fee_factor=fee_factor,
-        nonce=nonce,
-        chain_id=chain_id,
-    )
-    print(to_json(tx))
-
-
-governance_group.add_command(set_upgrade_manager_contract)
-
-
-@command()
-@rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_omission_accountability_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -981,7 +831,7 @@ def set_omission_accountability_contract(
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -998,18 +848,15 @@ def set_omission_accountability_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_omission_accountability_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("contract-address-str", metavar="CONTRACT-ADDRESS", nargs=1)
 def set_liquid_logic_contract(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -1028,7 +875,7 @@ def set_liquid_logic_contract(
     """
 
     contract_address = Web3.to_checksum_address(contract_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -1045,19 +892,102 @@ def set_liquid_logic_contract(
     print(to_json(tx))
 
 
-governance_group.add_command(set_liquid_logic_contract)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
+@tx_aux_options
+@argument("contract-address-str", metavar="CONTRACT-ADDRESS")
+def set_auctioneer_contract(
+    rpc_endpoint: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
+    from_str: Optional[str],
+    gas: Optional[str],
+    gas_price: Optional[str],
+    max_priority_fee_per_gas: Optional[str],
+    max_fee_per_gas: Optional[str],
+    fee_factor: Optional[float],
+    nonce: Optional[int],
+    chain_id: Optional[int],
+    contract_address_str: str,
+):
+    """
+    Set the Auctioneer contract address.
+
+    Restricted to the Operator account. See `setAuctioneerContract` on Autonity
+    contract.
+    """
+    contract_address = Web3.to_checksum_address(contract_address_str)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
+    aut = autonity_from_endpoint_arg(rpc_endpoint)
+
+    tx = create_contract_tx_from_args(
+        function=aut.set_auctioneer_contract(contract_address),
+        from_addr=from_addr,
+        gas=gas,
+        gas_price=gas_price,
+        max_fee_per_gas=max_fee_per_gas,
+        max_priority_fee_per_gas=max_priority_fee_per_gas,
+        fee_factor=fee_factor,
+        nonce=nonce,
+        chain_id=chain_id,
+    )
+    print(to_json(tx))
+
+
+@governance_group.command()
+@rpc_endpoint_option
+@from_options()
+@tx_aux_options
+@argument("contract-address-str", metavar="CONTRACT-ADDRESS")
+def set_upgrade_manager_contract(
+    rpc_endpoint: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
+    from_str: Optional[str],
+    gas: Optional[str],
+    gas_price: Optional[str],
+    max_priority_fee_per_gas: Optional[str],
+    max_fee_per_gas: Optional[str],
+    fee_factor: Optional[float],
+    nonce: Optional[int],
+    chain_id: Optional[int],
+    contract_address_str: str,
+):
+    """
+    Set the Upgrade Manager contract address.
+
+    Restricted to the Operator account. See `setUpgradeManagerContract` on Autonity
+    contract.
+    """
+    contract_address = Web3.to_checksum_address(contract_address_str)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
+    aut = autonity_from_endpoint_arg(rpc_endpoint)
+
+    tx = create_contract_tx_from_args(
+        function=aut.set_upgrade_manager_contract(contract_address),
+        from_addr=from_addr,
+        gas=gas,
+        gas_price=gas_price,
+        max_fee_per_gas=max_fee_per_gas,
+        max_priority_fee_per_gas=max_priority_fee_per_gas,
+        fee_factor=fee_factor,
+        nonce=nonce,
+        chain_id=chain_id,
+    )
+    print(to_json(tx))
+
+
+@governance_group.command()
+@rpc_endpoint_option
+@from_options()
 @tx_aux_options
 @argument("amount-str", metavar="AMOUNT", nargs=1)
 @argument("recipient-str", metavar="RECIPIENT", required=False)
 def mint(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -1070,14 +1000,14 @@ def mint(
     recipient_str: Optional[str],
 ) -> None:
     """
-    Mint new stake token (NTN) and add it to the recipient balance.
+    Mint new stake token (NTN) and add it to the recipient's balance.
 
     If recipient is not specified, the caller's address is used.
     Restricted to the Operator account. See `mint` on Autonity contract.
     """
 
     token_units = parse_newton_value_representation(amount_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     recipient = Web3.to_checksum_address(recipient_str) if recipient_str else from_addr
 
     aut = autonity_from_endpoint_arg(rpc_endpoint)
@@ -1096,19 +1026,16 @@ def mint(
     print(to_json(tx))
 
 
-governance_group.add_command(mint)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("amount-str", metavar="AMOUNT")
 @argument("account-str", metavar="ACCOUNT", required=False)
 def burn(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -1129,7 +1056,7 @@ def burn(
     """
 
     token_units = parse_newton_value_representation(amount_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     account = Web3.to_checksum_address(account_str) if account_str else from_addr
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
@@ -1147,18 +1074,15 @@ def burn(
     print(to_json(tx))
 
 
-governance_group.add_command(burn)
-
-
-@command()
+@governance_group.command()
 @rpc_endpoint_option
-@keyfile_option()
-@from_option
+@from_options()
 @tx_aux_options
 @argument("slasher-address-str", metavar="SLASHER-ADDRESS")
 def set_slasher(
     rpc_endpoint: Optional[str],
     keyfile: Optional[str],
+    trezor: Optional[str],
     from_str: Optional[str],
     gas: Optional[str],
     gas_price: Optional[str],
@@ -1175,7 +1099,7 @@ def set_slasher(
     Restricted to the Operator account. See `setSlasher` on Autonity contract.
     """
     slasher_address = Web3.to_checksum_address(slasher_address_str)
-    from_addr = from_address_from_argument(from_str, keyfile)
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
     aut = autonity_from_endpoint_arg(rpc_endpoint)
 
     tx = create_contract_tx_from_args(
@@ -1192,4 +1116,137 @@ def set_slasher(
     print(to_json(tx))
 
 
-governance_group.add_command(set_slasher)
+@governance_group.command()
+@rpc_endpoint_option
+@from_options()
+@tx_aux_options
+@argument("min-base-fee", type=int)
+@argument("base-fee-change-denominator", type=int)
+@argument("elasticity-multiplier", type=int)
+@argument("gas-limit-bound-divisor", type=int)
+def set_eip1559_params(
+    rpc_endpoint: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
+    from_str: Optional[str],
+    gas: Optional[str],
+    gas_price: Optional[str],
+    max_priority_fee_per_gas: Optional[str],
+    max_fee_per_gas: Optional[str],
+    fee_factor: Optional[float],
+    nonce: Optional[int],
+    chain_id: Optional[int],
+    min_base_fee: int,
+    base_fee_change_denominator: int,
+    elasticity_multiplier: int,
+    gas_limit_bound_divisor: int,
+):
+    """
+    Set the EIP-1559 parameters for the next epoch.
+
+    Restricted to the Operator account. See `setEip1559Params` on Autonity contract.
+    """
+    eip_params = Eip1559(
+        min_base_fee=min_base_fee,
+        base_fee_change_denominator=base_fee_change_denominator,
+        elasticity_multiplier=elasticity_multiplier,
+        gas_limit_bound_divisor=gas_limit_bound_divisor,
+    )
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
+    aut = autonity_from_endpoint_arg(rpc_endpoint)
+
+    tx = create_contract_tx_from_args(
+        function=aut.set_eip1559_params(eip_params),
+        from_addr=from_addr,
+        gas=gas,
+        gas_price=gas_price,
+        max_fee_per_gas=max_fee_per_gas,
+        max_priority_fee_per_gas=max_priority_fee_per_gas,
+        fee_factor=fee_factor,
+        nonce=nonce,
+        chain_id=chain_id,
+    )
+    print(to_json(tx))
+
+
+@governance_group.command()
+@rpc_endpoint_option
+@from_options()
+@tx_aux_options
+@argument("threshold", type=int)
+def set_clustering_threshold(
+    rpc_endpoint: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
+    from_str: Optional[str],
+    gas: Optional[str],
+    gas_price: Optional[str],
+    max_priority_fee_per_gas: Optional[str],
+    max_fee_per_gas: Optional[str],
+    fee_factor: Optional[float],
+    nonce: Optional[int],
+    chain_id: Optional[int],
+    threshold: int,
+):
+    """
+    Set the clustering threshold for consensus messaging.
+
+    Restricted to the Operator account. See `setClusteringThreshold` on Autonity
+    contract.
+    """
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
+    aut = autonity_from_endpoint_arg(rpc_endpoint)
+
+    tx = create_contract_tx_from_args(
+        function=aut.set_clustering_threshold(threshold),
+        from_addr=from_addr,
+        gas=gas,
+        gas_price=gas_price,
+        max_fee_per_gas=max_fee_per_gas,
+        max_priority_fee_per_gas=max_priority_fee_per_gas,
+        fee_factor=fee_factor,
+        nonce=nonce,
+        chain_id=chain_id,
+    )
+    print(to_json(tx))
+
+
+@governance_group.command()
+@rpc_endpoint_option
+@from_options()
+@tx_aux_options
+@argument("gas_limit", type=int)
+def set_gas_limit(
+    rpc_endpoint: Optional[str],
+    keyfile: Optional[str],
+    trezor: Optional[str],
+    from_str: Optional[str],
+    gas: Optional[str],
+    gas_price: Optional[str],
+    max_priority_fee_per_gas: Optional[str],
+    max_fee_per_gas: Optional[str],
+    fee_factor: Optional[float],
+    nonce: Optional[int],
+    chain_id: Optional[int],
+    gas_limit: int,
+):
+    """
+    Set the gas limit.
+
+    Restricted to the Operator account. See `setGasLimit` on Autonity contract.
+    """
+    from_addr = validate_authenticator_account(from_str, keyfile=keyfile, trezor=trezor)
+    aut = autonity_from_endpoint_arg(rpc_endpoint)
+
+    tx = create_contract_tx_from_args(
+        function=aut.set_gas_limit(gas_limit),
+        from_addr=from_addr,
+        gas=gas,
+        gas_price=gas_price,
+        max_fee_per_gas=max_fee_per_gas,
+        max_priority_fee_per_gas=max_priority_fee_per_gas,
+        fee_factor=fee_factor,
+        nonce=nonce,
+        chain_id=chain_id,
+    )
+    print(to_json(tx))
